@@ -1,5 +1,6 @@
 import functools
 import logging
+import os
 import sys
 import re
 
@@ -13,9 +14,9 @@ from wikirate4py.exceptions import IllegalHttpMethod, BadRequestException, Unaut
     ForbiddenException, NotFoundException, TooManyRequestsException, WikiRateServerErrorException, HTTPException, \
     WikiRate4PyException
 from wikirate4py.models import (Company, Topic, Metric, ResearchGroup, CompanyGroup, Source, CompanyItem, MetricItem,
-                                Answer,
-                                ResearchGroupItem, RelationshipAnswer, SourceItem, TopicItem, AnswerItem,
-                                CompanyGroupItem, RelationshipAnswerItem, Region, Project, ProjectItem)
+                                Answer, ResearchGroupItem, RelationshipAnswer, SourceItem, TopicItem, AnswerItem,
+                                CompanyGroupItem, RelationshipAnswerItem, Region, Project, ProjectItem, RegionItem,
+                                Dataset, DatasetItem)
 
 log = logging.getLogger(__name__)
 
@@ -52,12 +53,11 @@ class API(object):
     @property
     def headers(self):
         headers = {
-            "content-type": "application/json",
             'X-API-Key': self.oauth_token
         }
         return headers
 
-    def request(self, method, path, headers, params):
+    def request(self, method, path, headers, params, files={}):
         method = method.strip().lower()
         if method not in self.allowed_methods:
             msg = "The '{0}' method is not accepted by the WikiRate " \
@@ -66,7 +66,11 @@ class API(object):
 
         # if an error was returned throw an exception
         try:
-            response = self.session.request(method, path, auth=self.auth, params=params, headers=headers)
+            response = self.session.request(method, path, auth=self.auth, data=params, headers=headers, timeout=120,
+                                            files=files)
+
+            if files.get("card[subcards][+file][file]") is not None:
+                files.get("card[subcards][+file][file]").close()
         except Exception as e:
             raise WikiRate4PyException(f'Failed to send request: {e}').with_traceback(sys.exc_info()[2])
         finally:
@@ -114,9 +118,9 @@ class API(object):
         path = self.format_path(path, self.wikirate_api_url)
         return self.request('get', path, headers=headers, params=params or {})
 
-    def post(self, path, params={}):
+    def post(self, path, params={}, files={}):
         path = self.format_path(path, self.wikirate_api_url)
-        return self.request('post', path, headers=self.headers, params=params or {})
+        return self.request('post', path, headers=self.headers, params=params or {}, files=files)
 
     def delete(self, path, params={}):
         path = self.format_path(path, self.wikirate_api_url)
@@ -131,6 +135,12 @@ class API(object):
         if path.startswith("/"):
             return urljoin(wikirate_api_url, path.lstrip('/'))
 
+    def list_to_str(self, list):
+        value_str = ''
+        for t in list:
+            value_str += t + '\n'
+        return value_str
+
     @objectify(Company)
     def get_company(self, identifier) -> Company:
         """get_company(identifier)
@@ -149,7 +159,9 @@ class API(object):
         if isinstance(identifier, int):
             return self.get("/~{0}.json".format(identifier))
         else:
-            return self.get("/{0}.json".format(identifier.replace(" ", "_")))
+            return self.get("/{0}.json".format(
+                identifier.replace(',', ' ').replace('.', ' ').replace('/', ' ').replace('-', ' ').strip().replace(" ",
+                                                                                                                   "_")))
 
     @objectify(CompanyItem, list=True)
     def get_companies(self, **kwargs):
@@ -170,7 +182,7 @@ class API(object):
 
         """
         return self.get("/Company.json", endpoint_params=('limit', 'offset'),
-                        filters=('project', 'name', 'company_group', 'country', 'bookmark'),
+                        filters=('name', 'company_category', 'company_group', 'country'),
                         **kwargs)
 
     @objectify(Topic)
@@ -191,7 +203,9 @@ class API(object):
         if isinstance(identifier, int):
             return self.get("/~{0}.json".format(identifier))
         else:
-            return self.get("/{0}.json".format(identifier.replace(" ", "_")))
+            return self.get("/{0}.json".format(
+                identifier.replace(',', ' ').replace('.', ' ').replace('/', ' ').replace('-', ' ').strip().replace(" ",
+                                                                                                                   "_")))
 
     @objectify(TopicItem, True)
     def get_topics(self, **kwargs):
@@ -211,36 +225,19 @@ class API(object):
         :py:class:`List`\[:class:`~wikirate4py.models.TopicItem`]
 
         """
-        return self.get("/Topics.json", endpoint_params=('limit', 'offset'), filters=('name',), **kwargs)
+        return self.get("/Topics.json", endpoint_params=('limit', 'offset'), filters=('name', 'bookmark'), **kwargs)
 
     @objectify(Metric)
-    def get_metric(self, identifier):
-        """get_metric(identifier)
-
-        Returns a WikiRate Metric based on the given identifier (name or number)
-
-        Parameters
-        ----------
-        identifier
-            two different identifiers are allowed for WikiRate entities, numerical identifiers or name identifiers
-
-        Returns
-        -------
-            :py:class:`~wikirate4py.models.Metric`
-        """
-        if isinstance(identifier, int):
-            return self.get("/~{0}.json".format(identifier))
-        else:
-            return self.get("/{0}.json".format(identifier.replace(" ", "_")))
-
-    @objectify(Metric)
-    def get_metric(self, metric_name, metric_designer):
+    def get_metric(self, identifier=None, metric_name=None, metric_designer=None):
         """get_metric(metric_name, metric_designer)
 
         Returns a WikiRate Metric based on the given metric name and metric designer.
 
         Parameters
         ----------
+        identifier
+            two different identifiers are allowed for WikiRate entities, numerical identifiers or name identifiers
+
         metric_name
             name of metric
 
@@ -252,7 +249,12 @@ class API(object):
         -------
             :py:class:`~wikirate4py.models.Metric`
         """
-        return self.get("/{0}+{1}.json".format(metric_designer.replace(" ", "_"), metric_name.replace(" ", "_")))
+        if isinstance(identifier, int):
+            return self.get("/~{0}.json".format(identifier))
+        elif isinstance(identifier, str):
+            return self.get("/{0}.json".format(identifier.replace(" ", "_")))
+        else:
+            return self.get("/{0}+{1}.json".format(metric_designer.replace(" ", "_"), metric_name.replace(" ", "_")))
 
     @objectify(MetricItem, list=True)
     def get_metrics(self, **kwargs):
@@ -272,7 +274,10 @@ class API(object):
             :py:class:`List`\[:class:`~wikirate4py.models.MetricItem`]
 
         """
-        return self.get("/Metrics.json", endpoint_params=('limit', 'offset'), filters=('name',), **kwargs)
+        return self.get("/Metrics.json", endpoint_params=('limit', 'offset'), filters=(
+            'name', 'bookmark', 'wikirate_topic', 'designer', 'published', 'metric_type', 'value_type',
+            'research_policy',
+            'dataset'), **kwargs)
 
     @objectify(ResearchGroup)
     def get_research_group(self, identifier):
@@ -292,7 +297,9 @@ class API(object):
         if isinstance(identifier, int):
             return self.get("/~{0}.json".format(identifier))
         else:
-            return self.get("/{0}.json".format(identifier.replace(" ", "_")))
+            return self.get("/{0}.json".format(
+                identifier.replace(',', ' ').replace('.', ' ').replace('/', ' ').replace('-', ' ').strip().replace(" ",
+                                                                                                                   "_")))
 
     @objectify(ResearchGroupItem, list=True)
     def get_research_groups(self, **kwargs):
@@ -331,7 +338,9 @@ class API(object):
         if isinstance(identifier, int):
             return self.get("/~{0}.json".format(identifier))
         else:
-            return self.get("/{0}.json".format(identifier.replace(" ", "_")))
+            return self.get("/{0}.json".format(
+                identifier.replace(',', ' ').replace('.', ' ').replace('/', ' ').replace('-', ' ').strip().replace(" ",
+                                                                                                                   "_")))
 
     @objectify(CompanyGroupItem, True)
     def get_company_groups(self, **kwargs):
@@ -371,7 +380,9 @@ class API(object):
         if isinstance(identifier, int):
             return self.get("/~{0}.json".format(identifier))
         else:
-            return self.get("/{0}.json".format(identifier.replace(" ", "_")))
+            return self.get("/{0}.json".format(
+                identifier.replace(',', ' ').replace('.', ' ').replace('/', ' ').replace('-', ' ').strip().replace(" ",
+                                                                                                                   "_")))
 
     @objectify(SourceItem, True)
     def get_sources(self, **kwargs):
@@ -385,12 +396,26 @@ class API(object):
             default value 0, the (zero-based) offset of the first item in the collection to return
         limit
             default value 20, the maximum number of entries to return. If the value exceeds the maximum, then the maximum value will be used.
+        company_name
+            filter sources where the company name matches fully or partially the given string
+        year
+            filter sources based on given year
+        wikirate_title
+            filter sources where their title match fully or partially the given string
+        report_type
+            filter sources based on the report type
+        wikirate_topic
+            filter sources based on given topic
+        wikirate_link
+            filter sources where their url matches fully or partially the given string
 
         Returns
         -------
             :py:class:`List`\[:class:`~wikirate4py.models.Source`]
         """
-        return self.get("/Sources.json", endpoint_params=('limit', 'offset'), **kwargs)
+        return self.get("/Sources.json", endpoint_params=('limit', 'offset'), filters=(
+            'name', 'wikirate_title', 'wikirate_topic', 'report_type', 'year', 'wikirate_link', 'company_name'),
+                        **kwargs)
 
     @objectify(Answer)
     def get_answer(self, id):
@@ -410,15 +435,15 @@ class API(object):
         return self.get("/~{0}.json".format(id))
 
     @objectify(AnswerItem, True)
-    def get_answers(self, id, **kwargs):
-        """get_answers(id, *, offset, limit, year, status, company_group, country, value, value_from, value_to, \
+    def get_answers_by_metric_id(self, metric_id, **kwargs):
+        """get_answers_by_metric_id(metric_id, *, offset, limit, year, status, company_group, country, company_id, value, value_from, value_to, \
                        updated, updater, outliers, source, verification, project, bookmark)
 
         Returns a list of WikiRate Answers
 
         Parameters
         ----------
-        id
+        metric_id
             numeric metric identifier
 
         offset
@@ -439,6 +464,12 @@ class API(object):
         country
             country name, restricts to answers with companies located in the specified country
 
+        company_id
+            company identifier, restricts to answers of the defined company
+
+        company_name
+            restricts to answers of the defined company name
+
         value
             answer value to match
 
@@ -482,14 +513,15 @@ class API(object):
         -------
         :py:class:`List`\[:class:`~wikirate4py.models.AnswerItem`]
         """
-        return self.get("/~{0}+Answer.json".format(id), endpoint_params=('limit', 'offset'),
-                        filters=(
-                            'year', 'status', 'company_group', 'country', 'value', 'value_from', 'value_to', 'updated',
-                            'updater', 'outliers', 'source', 'verification', 'project', 'bookmark'), **kwargs)
+        return self.get("/~{0}+Answer.json".format(metric_id), endpoint_params=('limit', 'offset'),
+                        filters=('year', 'status', 'company_group', 'country', 'value', 'value_from', 'value_to',
+                                 'updated', 'company_id', 'company_name', 'dataset', 'updater', 'outliers', 'source',
+                                 'verification', 'bookmark', 'published'),
+                        **kwargs)
 
     @objectify(AnswerItem, True)
     def get_answers(self, metric_name, metric_designer, **kwargs):
-        """get_answers(metric_name, metric_designer, *, offset, limit, year, status, company_group, country, value, value_from, value_to, \
+        """get_answers(metric_name, metric_designer, *, company_name, company_id, offset, limit, year, status, company_group, country, value, value_from, value_to,
                        updated, updater, outliers, source, verification, project, bookmark)
 
         Returns a list of WikiRate Answers
@@ -501,6 +533,12 @@ class API(object):
         metric_designer
             name of relationship metric designer
 
+        company_name
+            restrict the answers based on the given company
+
+        company_id
+            restrict the answers based on the given company
+
         offset
             default value 0, the (zero-based) offset of the first item in the collection to return
 
@@ -557,6 +595,11 @@ class API(object):
         bookmark
             - `bookmark`, restrict to answers you have bookmarked
             - `nobookmark`, restrict to answers you have not bookmarked
+        published
+            - `true`, returns only published answers (default mode)
+            - `false`, returns only unpublished answers
+            - `all`, returns all published and unpublished answers
+
 
         Returns
         -------
@@ -564,11 +607,11 @@ class API(object):
 
         """
         return self.get(
-            "/~{0}+{1}+Answer.json".format(metric_designer.replace(" ", "_"), metric_name.replace(" ", "_")),
+            "/{0}+{1}+Answer.json".format(metric_designer.replace(" ", "_"), metric_name.replace(" ", "_")),
             endpoint_params=('limit', 'offset'),
-            filters=(
-                'year', 'status', 'company_group', 'country', 'value', 'value_from', 'value_to', 'updated',
-                'updater', 'outliers', 'source', 'verification', 'project', 'bookmark'), **kwargs)
+            filters=('year', 'status', 'company_group', 'country', 'value', 'value_from', 'value_to', 'updated',
+                     'company_id', 'company_name', 'dataset', 'updater', 'outliers', 'source', 'verification',
+                     'bookmark', 'published'), **kwargs)
 
     @objectify(RelationshipAnswer)
     def get_relationship_answer(self, id):
@@ -588,8 +631,8 @@ class API(object):
         return self.get("/~{0}.json".format(id))
 
     @objectify(RelationshipAnswerItem, True)
-    def get_relationship_answers(self, id, **kwargs):
-        """get_relationship_answers(id, *, offset, limit, year, status, company_group, country, value, value_from, value_to, \
+    def get_relationship_answers_by_metric_id(self, metric_id, **kwargs):
+        """get_relationship_answers_by_metric_id(metric_id, *, offset, limit, year, status, company_group, country, value, value_from, value_to, \
                        updated, updater, outliers, source, verification, project, bookmark)
 
         Returns a list of WikiRate Relationship Answers
@@ -656,15 +699,20 @@ class API(object):
                 - `bookmark`, restrict to relationship answers you have bookmarked
                 - `nobookmark`, restrict to relationship answers you have not bookmarked
 
+            published
+            - `true`, returns only published answers (default mode)
+            - `false`, returns only unpublished answers
+            - `all`, returns all published and unpublished answers
+
             Returns
             -------
                 :py:class:`List`\[:class:`~wikirate4py.models.RelationshipAnswerItem`]
 
             """
-        return self.get("/~{0}+Relationship_Answer.json".format(id),
+        return self.get("/~{0}+Relationship_Answer.json".format(metric_id),
                         endpoint_params=('limit', 'offset'), filters=(
                 'year', 'status', 'company_group', 'country', 'value', 'value_from', 'value_to', 'updated',
-                'updater', 'outliers', 'source', 'verification', 'project', 'bookmark'), **kwargs)
+                'updater', 'outliers', 'source', 'verification', 'project', 'bookmark', 'published'), **kwargs)
 
     @objectify(RelationshipAnswerItem, True)
     def get_relationship_answers(self, metric_name, metric_designer, **kwargs):
@@ -734,6 +782,11 @@ class API(object):
                 - `bookmark`, restrict to relationship answers you have bookmarked
                 - `nobookmark`, restrict to relationship answers you have not bookmarked
 
+            published
+            - `true`, returns only published answers (default mode)
+            - `false`, returns only unpublished answers
+            - `all`, returns all published and unpublished answers
+
             Returns
             -------
                 :py:class:`List`\[:class:`~wikirate4py.models.RelationshipAnswerItem`]
@@ -744,7 +797,7 @@ class API(object):
                                                                     metric_name.replace(" ", "_")),
                         endpoint_params=('limit', 'offset'), filters=(
                 'year', 'status', 'company_group', 'country', 'value', 'value_from', 'value_to', 'updated',
-                'updater', 'outliers', 'source', 'verification', 'project', 'bookmark'), **kwargs)
+                'updater', 'outliers', 'source', 'verification', 'project', 'bookmark', 'published'), **kwargs)
 
     @objectify(Project)
     def get_project(self, identifier):
@@ -764,7 +817,9 @@ class API(object):
         if isinstance(identifier, int):
             return self.get("/~{0}.json".format(identifier))
         else:
-            return self.get("/{0}.json".format(identifier.replace(" ", "_")))
+            return self.get("/{0}.json".format(
+                identifier.replace(',', ' ').replace('.', ' ').replace('/', ' ').replace('-', ' ').strip().replace(" ",
+                                                                                                                   "_")))
 
     @objectify(ProjectItem, True)
     def get_projects(self, **kwargs):
@@ -784,9 +839,53 @@ class API(object):
             :py:class:`List`\[:class:`~wikirate4py.models.ProjectItem`]
 
         """
-        return self.get("/Projects.json", endpoint_params=('limit', 'offset'), filters=('name',), **kwargs)
+        return self.get("/Projects.json", endpoint_params=('limit', 'offset'), filters=('name', 'wikirate_status'),
+                        **kwargs)
 
-    @objectify(Region, True)
+    @objectify(Dataset)
+    def get_dataset(self, identifier):
+        """get_dataset(identifier)
+        Returns a WikiRate Dataset based on the given identifier (name or number)
+
+        Parameters
+        ----------
+        identifier
+            two different identifiers are allowed for WikiRate entities, numerical identifiers or name identifiers
+
+        Returns
+        -------
+            :py:class:`~wikirate4py.models.Dataset`
+
+        """
+        if isinstance(identifier, int):
+            return self.get("/~{0}.json".format(identifier))
+        else:
+            return self.get("/{0}.json".format(
+                identifier.replace(',', ' ').replace('.', ' ').replace('/', ' ').replace('-', ' ').strip().replace(" ",
+                                                                                                                   "_")))
+
+    @objectify(DatasetItem, True)
+    def get_datasets(self, **kwargs):
+        """get_datasets(*, offset, limit)
+
+        Returns a list of WikiRate Datasets
+
+        Parameters
+        -------------------
+        offset
+            default value 0, the (zero-based) offset of the first item in the collection to return
+        limit
+            default value 20, the maximum number of entries to return. If the value exceeds the maximum, then the maximum value will be used.
+
+        Returns
+        -------
+            :py:class:`List`\[:class:`~wikirate4py.models.DatasetItem`]
+
+        """
+        return self.get("/Data_sets.json", endpoint_params=('limit', 'offset'), filters=('name', 'wikirate_topic'),
+                        **kwargs)
+
+    @objectify(RegionItem, True)
     def get_regions(self, **kwargs):
         """get_regions(*, offset, limit)
 
@@ -806,6 +905,26 @@ class API(object):
         """
         return self.get("/Region.json", endpoint_params=('limit', 'offset'), **kwargs)
 
+    @objectify(Region)
+    def get_region(self, identifier):
+        """get_project(identifier)
+        Returns a WikiRate Region based on the given identifier (name or number)
+
+        Parameters
+        ----------
+        identifier
+            two different identifiers are allowed for WikiRate entities, numerical identifiers or name identifiers
+
+        Returns
+        -------
+            :py:class:`~wikirate4py.models.Project`
+
+        """
+        if isinstance(identifier, int):
+            return self.get("/~{0}.json".format(identifier))
+        else:
+            return self.get("/{0}.json".format(identifier.replace(" ", "_")))
+
     def search_by_name(self, entity, name, **kwargs):
         """search_by_name(entity, name, *, offset, limit)
 
@@ -854,6 +973,8 @@ class API(object):
             return self.get_research_groups(name=name, **kwargs)
         elif entity is Project:
             return self.get_projects(name=name, **kwargs)
+        else:
+            raise WikiRate4PyException(f"Type of parameter 'entity' ({type(entity)}) is not allowed")
 
     @objectify(SourceItem, True)
     def search_source_by_url(self, url, **kwargs):
@@ -919,6 +1040,7 @@ class API(object):
             "card[type]": "Company",
             "card[name]": name,
             "card[subcards][+headquarters]": headquarters,
+            "confirmed": "true",
             "format": "json",
             "success[format]": "json"
         }
@@ -936,6 +1058,35 @@ class API(object):
 
         return self.post("/card/create", params)
 
+    @objectify(RegionItem, True)
+    def update_headquarters(self, identifier, headquarters):
+        params = {
+            "card[content]": headquarters,
+            "format": "json",
+            "success[format]": "json"
+        }
+        if isinstance(identifier, int):
+            return self.post("/update/~{0}".format(identifier) + '+:headquarters', params)
+        else:
+            return self.post("/update/{0}".format(
+                identifier.replace(',', ' ').replace('.', ' ').replace('/', ' ').replace('-', ' ').strip().replace(" ",
+                                                                                                                   "_")) + '+:headquarters',
+                             params)
+
+    def update_oc_company_number(self, identifier, company_number):
+        params = {
+            "card[content]": company_number,
+            "format": "json",
+            "success[format]": "json"
+        }
+        if isinstance(identifier, int):
+            return self.post("/update/~{0}".format(identifier) + '+OpenCorporates', params)
+        else:
+            return self.post("/update/{0}".format(
+                identifier.replace(',', ' ').replace('.', ' ').replace('/', ' ').replace('-', ' ').strip().replace(" ",
+                                                                                                                   "_")) + '+OpenCorporates',
+                             params)
+
     @objectify(Answer)
     def add_research_metric_answer(self, **kwargs):
         """add_research_metric_answer(metric_designer, metric_name, company, year, value, source, *, comment)
@@ -1055,11 +1206,74 @@ class API(object):
 
         return self.post("/card/update", params)
 
+    @objectify(Answer)
+    def update_research_metric_answer_by_id(self, **kwargs):
+        """update_research_metric_answer(id, *, company, year, value, source, comment)
+
+        Updates and Returns an existing metric answer
+
+        Parameters
+        ----------
+        id
+            answer's id
+
+        metric_name
+            name of metric
+
+        metric_designer
+            name of metric designer
+
+        company
+            company the answer is assigned to
+
+        year
+            reporting year
+
+        value
+            new value
+
+        source
+            new source name
+
+        comment
+            new comment on the metric answer
+
+        Returns
+        -------
+            :py:class:`~wikirate4py.models.Answer`
+
+        """
+        required_param = 'id'
+        optional_params = ('company', 'year', 'value', 'source', 'comment')
+
+        if required_param not in kwargs:
+            raise WikiRate4PyException(
+                """Invalid set of params! You need to define all the following params to update the research answer: """ + required_param.__str__())
+
+        if kwargs.get('company') is not None:
+            company_identifier = '~' + str(kwargs['company']) if isinstance(kwargs['company'], int) else kwargs[
+                'company'].replace(' ', '_')
+        params = {
+            "card[type]": "Answer",
+            "card[name]": '~' + kwargs['id'].__str__(),
+            "format": "json",
+            "success[format]": "json"
+        }
+        for k in kwargs.keys():
+            if k == 'comment':
+                params['card[subcards][+:discussion]'] = str(kwargs[k])
+            elif k != required_param and k in optional_params:
+                params['card[subcards][+:' + k + ']'] = str(kwargs[k])
+
+        log.debug("PARAMS: %r", params)
+
+        return self.post("/card/update", params)
+
     @objectify(RelationshipAnswer)
     def add_relationship_metric_answer(self, **kwargs):
         """add_relationship_metric_answer(metric_designer, metric_name, company, year, value, source, *, comment)
 
-        Updates and Returns an existing relationship metric answer
+        Adds and Returns a relationship metric answer
 
         Parameters
         ----------
@@ -1120,6 +1334,76 @@ class API(object):
 
         return self.post("/card/create", params)
 
+    @objectify(Metric)
+    def add_metric(self, **kwargs):
+        """add_metric(designer, name, about, methodology, topics, value_type, options, research_policy, report_type)
+
+        Creates and Returns a new Metric
+
+        Parameters
+        -------------------
+        designer
+            designer of the metric
+
+        name
+            name of the metric
+
+        about
+            about the metric (plain text/html can be given as input)
+
+        methodology
+            metric's methodology (plain text/html can be given as input)
+
+        topics
+            a list of metrics
+
+        value_type
+            value type
+
+        unit
+            unit can be given (e.g. USD, EURO, Square meters etc)
+
+        value_options
+            a list of value options (if the value type is Multi-Category)
+
+        research_policy
+            research policy's options: Community Assessed, Designer Assessed
+
+        report_type
+            report type
+
+        Returns
+        -------
+            :py:class:`~wikirate4py.models.Metric`
+
+        """
+        required_params = ['designer', 'name', 'value_type']
+        optional_params = ('about', 'methodology', 'unit', 'topics', 'value_options', 'research_policy', 'report_type')
+
+        for k in required_params:
+            if k not in kwargs:
+                raise WikiRate4PyException("""Invalid set of params! You need to define all the following params to create
+                                a new metric in WikiRate platform: """ + required_params.__str__())
+
+        params = {
+            "card[type]": "Metric",
+            "card[name]": kwargs['designer'] + '+' + kwargs['name'],
+            "card[subcards][+value_type]": kwargs['value_type'],
+            "card[skip]": "requirements",
+            "format": "json",
+            "success[format]": "json"
+        }
+
+        for k in optional_params:
+            if k in kwargs.keys():
+                if k in ['topics', 'value_options']:
+                    params['card[subcards][+' + k + ']'] = self.list_to_str(kwargs[k])
+                else:
+                    params['card[subcards][+' + k + ']'] = str(kwargs[k])
+
+        log.debug("PARAMS: %r", params)
+        return self.post("/card/create", params=params)
+
     @objectify(RelationshipAnswer)
     def update_relationship_metric_answer(self, **kwargs):
         """update_relationship_metric_answer(metric_designer, metric_name, company, year, value, source)
@@ -1191,15 +1475,18 @@ class API(object):
 
     @objectify(Source)
     def add_source(self, **kwargs):
-        """add_source(url, title, company, report_type, year)
+        """add_source(link, title, company, report_type, year)
 
         Updates and Returns an existing relationship metric answer
 
         Parameters
         -------------------
-        url
+        link
             url of the original source
 
+        file
+           file path to the file you want to upload as a source
+
         title
             give a title on the source
 
@@ -1211,40 +1498,58 @@ class API(object):
 
         year
             reporting year
+        file
+            filepath on the file you want to upload
 
         Returns
         -------
             :py:class:`~wikirate4py.models.Source`
 
         """
-        required_params = ('url', 'title')
-        optional_params = ('company', 'report_type', 'year')
+        required_params = ['title']
+        optional_params = ('link', 'company', 'report_type', 'year', 'file')
 
         for k in required_params:
             if k not in kwargs:
                 raise WikiRate4PyException("""Invalid set of params! You need to define all the following params to import 
-                        a new source in WikiRate platform: """ + required_params.__str__())
+                            a new source in WikiRate platform: """ + required_params.__str__())
+        if 'link' not in kwargs and 'file' not in kwargs:
+            raise WikiRate4PyException("""Invalid set of params! You need to define either a link or give a file path to 
+                                            upload a file while creating a new source: """ + required_params.__str__())
 
         params = {
             "card[type]": "Source",
             "card[subcards][+title]": kwargs['title'],
-            "card[subcards][+link]": kwargs['url'],
             "card[skip]": "requirements",
             "format": "json",
             "success[format]": "json"
         }
+        files = {}
         for k in kwargs.keys():
-            if k in optional_params:
+            if k in optional_params and k == "file":
+                data_file = open(os.path.realpath(kwargs[k]), 'rb')
+                files["card[subcards][+file][file]"] = data_file
+            else:
                 params['card[subcards][+' + k + ']'] = str(kwargs[k])
         log.debug("PARAMS: %r", params)
+        return self.post("/card/create", params=params, files=files)
 
-        return self.post("/card/create", params)
+    @objectify(Source)
+    def upload_file(self, source, file):
+        data_file = open(os.path.realpath(file), 'rb')
+        params = {
+            "format": "json",
+            "success[format]": "json"
+        }
+        files = {"card[subcards][+file][file]": data_file}
+
+        return self.post("/update/{0}".format(source), params=params, files=files)
 
     @objectify(Source)
     def update_source(self, **kwargs):
         """update_source(name, title, company, report_type, year)
 
-        Updates and Returns an existing relationship metric answer
+        Updates and Returns an existing source
 
         Parameters
         ----------
@@ -1290,9 +1595,23 @@ class API(object):
 
         return self.post("/card/update", params)
 
-    def delete_company(self, id):
-        """delete_company(id)
+    def delete_wikirate_entity(self, id):
+        """delete_wikirate_entity(id)
 
-        Deletes a company based on the given numeric identifier
+        Deletes a WikiRate entity based on the given numeric identifier
         """
         return self.delete("/~{0}".format(id))
+
+    def add_companies_to_group(self, group_id, list=[]):
+        ids = ""
+        for item in list:
+            ids += '~[[' + item + ']]\n'
+        params = {
+            "card[type]": "List",
+            "card[name]": '~' + group_id + '+' + 'Company',
+            "card[content]": ids,
+            "format": "json",
+            "success[format]": "json"
+        }
+
+        return self.post("/card/update", params)
